/*
 * say_de.c
 */

/*! \brief  say_number_full_de: German syntax */
/* New files:
 In addition to English, the following sounds are required:
 "millions"
 "1-and" through "9-and" 
 "1F" (eine)
 "1N" (ein)
 NB "1" is recorded as 'eins'
 */
static int say_number_full_de(const say_args_t *a, int num, const char *options)
{
	int res = 0, t = 0;
	int mf = 1;                            /* +1 = male and neuter; -1 = female */
	char fn[256] = "";
	char fna[256] = "";
	if (!num) 
		return say_digits_full(a, 0);

	if (options && (!strncasecmp(options, "f",1)))
		mf = -1;

	while(!res && num) {
		/* The grammar for German numbers is the same as for English except
		* for the following:
		* - numbers 20 through 99 are said in reverse order, i.e. 21 is
		*   "one-and twenty" and 68 is "eight-and sixty".
		* - "one" varies according to gender
		* - 100 is 'hundert', however all other instances are 'ein hundert'
		* - 1000 is 'tausend', however all other instances are 'ein tausend'
		* - 1000000 is always 'eine million'
		* - "million" is different in singular and plural form
		*/
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (num < 100 && t) {
			snprintf(fn, sizeof(fn), "digits/and");
			t = 0;
		} else if (num == 1 && mf == -1) {
			snprintf(fn, sizeof(fn), "digits/%dF", num);
			num = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			int ones = num % 10;
			if (ones) {
				snprintf(fn, sizeof(fn), "digits/%d-and", ones);
				num -= ones;
			} else {
				snprintf(fn, sizeof(fn), "digits/%d", num);
				num = 0;
			}
		} else if (num == 100 && t == 0) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			num = 0;
		} else if (num < ONE_THOUSAND) {
			int hundreds = num / 100;
			num = num % 100;
			if (hundreds == 1) {
				snprintf(fn, sizeof(fn), "digits/1N");
			} else {
				snprintf(fn, sizeof(fn), "digits/%d", hundreds);
			}
			snprintf(fna, sizeof(fna), "digits/hundred");
			t = 1;
		} else if (num == ONE_THOUSAND && t == 0) {
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = 0;
		} else 	if (num < ONE_MILLION) {
			int thousands = num / ONE_THOUSAND;
			num = num % ONE_THOUSAND;
			t = 1;
			if (thousands == 1) {
				snprintf(fn, sizeof(fn), "digits/1N");
				snprintf(fna, sizeof(fna), "digits/thousand");
			} else {
				res = say_number_full_de(a, thousands, options);
				if (res)
					return res;
				snprintf(fn, sizeof(fn), "digits/thousand");
			}
		} else if (num < ONE_BILLION) {
			int millions = num / ONE_MILLION;
			num = num % ONE_MILLION;
			t = 1;
			if (millions == 1) {
				snprintf(fn, sizeof(fn), "digits/1F");
				snprintf(fna, sizeof(fna), "digits/million");
			} else {
				res = say_number_full_de(a, millions, options);
				if (res)
					return res;
				snprintf(fn, sizeof(fn), "digits/millions");
			}
		} else if (num <= INT_MAX) {
			int billions = num / ONE_BILLION;
			num = num % ONE_BILLION;
			t = 1;
			if (billions == 1) {
				snprintf(fn, sizeof(fn), "digits/1F");
				snprintf(fna, sizeof(fna), "digits/milliard");
			} else {
				res = say_number_full_de(a, billions, options);
				if (res) {
					return res;
				}
				snprintf(fn, sizeof(fn), "digits/milliards");
			}
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res) {
			res = s_streamwait3(a, fn);
			if (!res) {
				if (!ast_strlen_zero(fna))
					res = s_streamwait3(a, fna);
				strcpy(fna, "");
			}
		}
	}
	return res;
}

/*! \brief  say_enumeration_full_de: German syntax */
static int say_enumeration_full_de(const say_args_t *a, int num,
	const char *options)
{
	/* options can be: '' or 'm' male gender; 'f' female gender; 'n' neuter gender; 'p' plural */
	int res = 0, t = 0;
	char fn[256] = "", fna[256] = "";
	char *gender;

	if (options && !strncasecmp(options, "f",1)) {
		gender = "F";
	} else if (options && !strncasecmp(options, "n",1)) {
		gender = "N";
	} else {
		gender = "";
	}

	if (!num) 
		return say_digits_full(a, 0);

	while(!res && num) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus"); /* kind of senseless for enumerations, but our best effort for error checking */
			num = negate(num);
		} else if (num < 100 && t) {
			snprintf(fn, sizeof(fn), "digits/and");
			t = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/h-%d%s", num, gender);
			num = 0;
		} else if (num < 100) {
			int ones = num % 10;
			if (ones) {
				snprintf(fn, sizeof(fn), "digits/%d-and", ones);
				num -= ones;
			} else {
				snprintf(fn, sizeof(fn), "digits/h-%d%s", num, gender);
				num = 0;
			}
		} else if (num == 100 && t == 0) {
			snprintf(fn, sizeof(fn), "digits/h-hundred%s", gender);
			num = 0;
		} else if (num < ONE_THOUSAND) {
			int hundreds = num / 100;
			num = num % 100;
			if (hundreds == 1) {
				snprintf(fn, sizeof(fn), "digits/1N");
			} else {
				snprintf(fn, sizeof(fn), "digits/%d", hundreds);
			}
			if (num) {					
				snprintf(fna, sizeof(fna), "digits/hundred");
			} else {
				snprintf(fna, sizeof(fna), "digits/h-hundred%s", gender);
			}
			t = 1;
		} else 	if (num < ONE_MILLION) {
			int thousands = num / ONE_THOUSAND;
			num = num % ONE_THOUSAND;
			if (thousands == 1) {
				if (num) {
					snprintf(fn, sizeof(fn), "digits/1N");
					snprintf(fna, sizeof(fna), "digits/thousand");
				} else {
					if (t) {
						snprintf(fn, sizeof(fn), "digits/1N");
						snprintf(fna, sizeof(fna), "digits/h-thousand%s", gender);
					} else {
						snprintf(fn, sizeof(fn), "digits/h-thousand%s", gender);
					}
				}
			} else {
				res = say_number_full_de(a, thousands, options);
				if (res) {
					return res;
				}
				if (num) {					
					snprintf(fn, sizeof(fn), "digits/thousand");
				} else {
					snprintf(fn, sizeof(fn), "digits/h-thousand%s", gender);
				}
			}
			t = 1;
		} else if (num < ONE_BILLION) {
			int millions = num / ONE_MILLION;
			num = num % ONE_MILLION;
			if (millions == 1) {
				if (num) {					
					snprintf(fn, sizeof(fn), "digits/1F");
					snprintf(fna, sizeof(fna), "digits/million");
				} else {
					snprintf(fn, sizeof(fn), "digits/1N");
					snprintf(fna, sizeof(fna), "digits/h-million%s", gender);
				}
			} else {
				res = say_number_full_de(a, millions, options);
				if (res) {
					return res;
				}
				if (num) {					
					snprintf(fn, sizeof(fn), "digits/millions");
				} else {
					snprintf(fn, sizeof(fn), "digits/h-million%s", gender);
				}
			}
			t = 1;
		} else if (num < INT_MAX) {
			int billions = num / ONE_BILLION;
			num = num % ONE_BILLION;
			if (billions == 1) {
				if (num) {
					snprintf(fn, sizeof(fn), "digits/1F");
					snprintf(fna, sizeof(fna), "digits/milliard");
				} else {
					snprintf(fn, sizeof(fn), "digits/1N");
					snprintf(fna, sizeof(fna), "digits/h-milliard%s", gender);
				}
			} else {
				res = say_number_full_de(a, billions, options);
				if (res)
					return res;
				if (num) {					
					snprintf(fn, sizeof(fna), "digits/milliards");
				} else {
					snprintf(fn, sizeof(fna), "digits/h-milliard%s", gender);
				}
			}
			t = 1;
		} else if (num == INT_MAX) {
			snprintf(fn, sizeof(fn), "digits/h-last%s", gender);
			num = 0;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}

		if (!res) {
			res = s_streamwait3(a, fn);
			if (!res) {
				if (strlen(fna) != 0) {
					res = s_streamwait3(a, fna);
				}
				strcpy(fna, "");
			}
		}
	}
	return res;
}

/* German syntax */
static int say_date_with_format_de(const say_args_t *a, const struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	say_date_with_format_f *f = say_date_with_format_de; /* shorthand for recursive call */

	for (offset=0 ; format[offset] != '\0' ; offset++) {
		switch (format[offset]) {
		case 'm':
			/* Month enumerated */
			res = say_enumeration_full_de(a, (tm.tm_mon + 1), "m");	
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */
			res = say_enumeration_full_de(a, tm.tm_mday, "m");	
			break;
		case 'Y':
			/* Year */
			{
				int year = tm.tm_year + 1900;
				if (year > 1999) {	/* year 2000 and later */
					res = say_number_full_de(a, year, NULL);	
				} else {
					if (year < 1100) {
						/* I'm not going to handle 1100 and prior */
						/* We'll just be silent on the year, instead of bombing out. */
					} else {
						/* year 1100 to 1999. will anybody need this?!? */
						/* say 1967 as 'neunzehn hundert sieben und sechzig' */
						res = s_streamwait(a, "digits/%d", (year / 100) );
						if (!res) {
							res = s_streamwait3(a, "digits/hundred");
							if (!res && year % 100 != 0) {
								res = say_number_full_de(a, year % 100, NULL);	
							}
						}
					}
				}
			}
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			res = s_streamwait(a, "digits/%d", hr_to_12(tm.tm_hour));
			if (!res)
				res = s_streamwait3(a, "digits/oclock");
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			res = say_number_full_de(a, tm.tm_hour, NULL);	
			if (!res)
				res = s_streamwait3(a, "digits/oclock");
			break;
		case 'M':
			/* Minute */
			if (tm.tm_min > 0 || format[offset+ 1 ] == 'S' ) { /* zero 'digits/0' only if seconds follow (kind of a hack) */
				res = say_number_full_de(a, tm.tm_min, "f");	
			}
			if ( !res && format[offset + 1] == 'S' ) { /* minutes only if seconds follow (kind of a hack) */
				if (tm.tm_min == 1) {
					res = s_streamwait3(a, "digits/minute");
				} else {
					res = s_streamwait3(a, "digits/minutes");
				}
			}
			break;
		case 'S':
			/* Seconds */
			res = s_streamwait3(a, "digits/and");
			if (!res) {
				res = say_number_full_de(a, tm.tm_sec, "f");	
				if (!res) {
					res = s_streamwait3(a, "digits/seconds");
				}
			}
			break;
		default:
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

static struct _lang_sw map_de = {
	.language = "de",
	.say_number_full = say_number_full_de,
	.say_enumeration_full = say_enumeration_full_de,
	.say_date_with_format = say_date_with_format_de,
	.date_fmt = "AdBY",
	.time_fmt = "HM",
	.datetime_fmt = "AdBYHM",
};

