/*
 * say_it.c
 */

/*! \brief  say_number_full_it:  Italian */
static int say_number_full_it(const say_args_t *a, int num, const char *options)
{
	int res = 0;

	if (!num)
		return say_digits_full(a, 0);

	/*
	Italian support

	digits/
		0..19, [2-9][018]	-> one-word numbers
		minus	-> 'meno'
		hundred	-> 'cento'
		thousand	-> 'mille'
		thousands	-> 'mila'
		million	-> 'un milione'
		millions	-> 'milioni'
	*/

	while ( res == 0 && num != 0) {
		const char *fn = NULL;
		char buf[256] = "";
		if (num < 0) {
			fn = "digits/minus";
			num = negate(num);
		} else if (num < ONE_HUNDRED) {
			/*
			 * Numbers up to 20, or numbers with units starting with
			 * a vowel, are pronounced as a single word. Others up to 99
			 * are pronounced as tens + units.
			 */
			int units = num % 10;
			if (num < 20 || units == 1 || units == 8)
				units = 0; /* play as single word and we are done */
			snprintf(buf, sizeof(buf), "digits/%d", num - units);
			fn = buf;
			num = units;
		} else if (num < ONE_THOUSAND) {
			/* 1xx -> 'cento' + xx, [2-9]xx -> [2-9] + 'cento' + xx */
			int hundreds = num / ONE_HUNDRED;
			if (hundreds > 1)
				res = say_number_full_it(a, hundreds, NULL);
			fn = "digits/hundred";	/* 'cento' */
			num %= ONE_HUNDRED;
		} else if (num < ONE_MILLION) {
			/* 1xxx -> 'mille' + xxx, [2..999]xxx -> [2..999] + 'mila' + xxx */
			int thousands = num/ONE_THOUSAND;
			if (thousands == 1)
				fn = "digits/thousand";		/* 'mille' */
			else {
				res = say_number_full_it(a, thousands, options);
				fn = "digits/thousands";	/* 'mila' */
			}
			num = num % ONE_THOUSAND;
		} else if (num < ONE_BILLION) {
			/* 1y -> 'un milione' + y, [2..999]y -> [2..999] + 'milioni' + y */
			int millions = num / ONE_MILLION;
			if (millions == 1)
				fn = "digits/million";		/* 'un milione' */
			else {
				res = say_number_full_it(a, millions, options);
				fn = "digits/millions";	/* 'milioni' */
			}
			num = num % ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			break;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

static int say_date_with_format_it(const say_args_t *a, const struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	say_date_with_format_f *f = say_date_with_format_it; /* shorthand for recursive call */
	char c;

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case 'm':	/* month, cardinal */
			res = say_number_full_it(a,  tm.tm_mon + 1, NULL);
			break;
		case 'd':
		case 'e':
			/* First day of the month is spelled as ordinal */
			if (tm.tm_mday == 1)
				res = s_streamwait3(a, "digits/h-1");
			else
				res = say_number_full_it(a, tm.tm_mday, NULL);
			break;
		case 'Y': /* Year */
			res = say_number_full_it(a, tm.tm_year + 1900, NULL);
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			/* XXX actually, mezzanotte, una, mezzogiorno... */
			res = s_streamwait(a, "digits/%d", hr_to_12(tm.tm_hour));
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			if (tm.tm_hour == 0) {
				res = s_streamwait3(a, "digits/ore-mezzanotte");
			} else if (tm.tm_hour == 1) {
				res = s_streamwait3(a, "digits/ore-una");
			} else {
				res = say_number_full_it(a, tm.tm_hour, NULL);
			}
			break;
		case 'M':
			/* Minute */
			res = say_number_full_it(a, tm.tm_min, NULL);
			break;
		case 'p':
			/* AM/PM */
			if (tm.tm_hour != 0 && tm.tm_hour != 12)
				res = s_streamwait3(a, hr_to_am_pm(tm.tm_hour));
			break;

		case 'S':
			/* Seconds */
			res = say_number_full_it(a, tm.tm_sec, NULL);
			break;

		case 'x':
			res = f(a, &tm, "AdBY", delta);
			break;

		default:
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

static struct _lang_sw map_it = {
	.language = "it",
	.say_number_full = say_number_full_it,
	.say_date_with_format = say_date_with_format_it,
	.date_fmt = "AdBY",
	.time_fmt = "HM", /* XXX missing 'e' between H and M */
	.datetime_fmt = "AdBYHM", /* XXX missing 'ore' between date and time */
};

